from visualization_msgs.srv._get_interactive_markers import GetInteractiveMarkers  # noqa: F401
from visualization_msgs.srv._get_interactive_markers import GetInteractiveMarkers_Event  # noqa: F401
from visualization_msgs.srv._get_interactive_markers import GetInteractiveMarkers_Request  # noqa: F401
from visualization_msgs.srv._get_interactive_markers import GetInteractiveMarkers_Response  # noqa: F401



# generated from rosidl_generator_py/resource/_idl.py.em
# with input from visualization_msgs:msg/Marker.idl
# generated code does not contain a copyright notice

# This is being done at the module level and not on the instance level to avoid looking
# for the same variable multiple times on each instance. This variable is not supposed to
# change during runtime so it makes sense to only look for it once.
from os import getenv

ros_python_check_fields = getenv('ROS_PYTHON_CHECK_FIELDS', default='')


# Import statements for member types

import builtins  # noqa: E402, I100

import rosidl_parser.definition  # noqa: E402, I100


class Metaclass_Marker(type):
    """Metaclass of message 'Marker'."""

    _CREATE_ROS_MESSAGE = None
    _CONVERT_FROM_PY = None
    _CONVERT_TO_PY = None
    _DESTROY_ROS_MESSAGE = None
    _TYPE_SUPPORT = None

    __constants = {
        'ARROW': 0,
        'CUBE': 1,
        'SPHERE': 2,
        'CYLINDER': 3,
        'LINE_STRIP': 4,
        'LINE_LIST': 5,
        'CUBE_LIST': 6,
        'SPHERE_LIST': 7,
        'POINTS': 8,
        'TEXT_VIEW_FACING': 9,
        'MESH_RESOURCE': 10,
        'TRIANGLE_LIST': 11,
        'ARROW_STRIP': 12,
        'ADD': 0,
        'MODIFY': 0,
        'DELETE': 2,
        'DELETEALL': 3,
    }

    @classmethod
    def __import_type_support__(cls):
        try:
            from rosidl_generator_py import import_type_support
            module = import_type_support('visualization_msgs')
        except ImportError:
            import logging
            import traceback
            logger = logging.getLogger(
                'visualization_msgs.msg.Marker')
            logger.debug(
                'Failed to import needed modules for type support:\n' +
                traceback.format_exc())
        else:
            cls._CREATE_ROS_MESSAGE = module.create_ros_message_msg__msg__marker
            cls._CONVERT_FROM_PY = module.convert_from_py_msg__msg__marker
            cls._CONVERT_TO_PY = module.convert_to_py_msg__msg__marker
            cls._TYPE_SUPPORT = module.type_support_msg__msg__marker
            cls._DESTROY_ROS_MESSAGE = module.destroy_ros_message_msg__msg__marker

            from builtin_interfaces.msg import Duration
            if Duration.__class__._TYPE_SUPPORT is None:
                Duration.__class__.__import_type_support__()

            from geometry_msgs.msg import Point
            if Point.__class__._TYPE_SUPPORT is None:
                Point.__class__.__import_type_support__()

            from geometry_msgs.msg import Pose
            if Pose.__class__._TYPE_SUPPORT is None:
                Pose.__class__.__import_type_support__()

            from geometry_msgs.msg import Vector3
            if Vector3.__class__._TYPE_SUPPORT is None:
                Vector3.__class__.__import_type_support__()

            from sensor_msgs.msg import CompressedImage
            if CompressedImage.__class__._TYPE_SUPPORT is None:
                CompressedImage.__class__.__import_type_support__()

            from std_msgs.msg import ColorRGBA
            if ColorRGBA.__class__._TYPE_SUPPORT is None:
                ColorRGBA.__class__.__import_type_support__()

            from std_msgs.msg import Header
            if Header.__class__._TYPE_SUPPORT is None:
                Header.__class__.__import_type_support__()

            from visualization_msgs.msg import MeshFile
            if MeshFile.__class__._TYPE_SUPPORT is None:
                MeshFile.__class__.__import_type_support__()

            from visualization_msgs.msg import UVCoordinate
            if UVCoordinate.__class__._TYPE_SUPPORT is None:
                UVCoordinate.__class__.__import_type_support__()

    @classmethod
    def __prepare__(cls, name, bases, **kwargs):
        # list constant names here so that they appear in the help text of
        # the message class under "Data and other attributes defined here:"
        # as well as populate each message instance
        return {
            'ARROW': cls.__constants['ARROW'],
            'CUBE': cls.__constants['CUBE'],
            'SPHERE': cls.__constants['SPHERE'],
            'CYLINDER': cls.__constants['CYLINDER'],
            'LINE_STRIP': cls.__constants['LINE_STRIP'],
            'LINE_LIST': cls.__constants['LINE_LIST'],
            'CUBE_LIST': cls.__constants['CUBE_LIST'],
            'SPHERE_LIST': cls.__constants['SPHERE_LIST'],
            'POINTS': cls.__constants['POINTS'],
            'TEXT_VIEW_FACING': cls.__constants['TEXT_VIEW_FACING'],
            'MESH_RESOURCE': cls.__constants['MESH_RESOURCE'],
            'TRIANGLE_LIST': cls.__constants['TRIANGLE_LIST'],
            'ARROW_STRIP': cls.__constants['ARROW_STRIP'],
            'ADD': cls.__constants['ADD'],
            'MODIFY': cls.__constants['MODIFY'],
            'DELETE': cls.__constants['DELETE'],
            'DELETEALL': cls.__constants['DELETEALL'],
        }

    @property
    def ARROW(self):
        """Message constant 'ARROW'."""
        return Metaclass_Marker.__constants['ARROW']

    @property
    def CUBE(self):
        """Message constant 'CUBE'."""
        return Metaclass_Marker.__constants['CUBE']

    @property
    def SPHERE(self):
        """Message constant 'SPHERE'."""
        return Metaclass_Marker.__constants['SPHERE']

    @property
    def CYLINDER(self):
        """Message constant 'CYLINDER'."""
        return Metaclass_Marker.__constants['CYLINDER']

    @property
    def LINE_STRIP(self):
        """Message constant 'LINE_STRIP'."""
        return Metaclass_Marker.__constants['LINE_STRIP']

    @property
    def LINE_LIST(self):
        """Message constant 'LINE_LIST'."""
        return Metaclass_Marker.__constants['LINE_LIST']

    @property
    def CUBE_LIST(self):
        """Message constant 'CUBE_LIST'."""
        return Metaclass_Marker.__constants['CUBE_LIST']

    @property
    def SPHERE_LIST(self):
        """Message constant 'SPHERE_LIST'."""
        return Metaclass_Marker.__constants['SPHERE_LIST']

    @property
    def POINTS(self):
        """Message constant 'POINTS'."""
        return Metaclass_Marker.__constants['POINTS']

    @property
    def TEXT_VIEW_FACING(self):
        """Message constant 'TEXT_VIEW_FACING'."""
        return Metaclass_Marker.__constants['TEXT_VIEW_FACING']

    @property
    def MESH_RESOURCE(self):
        """Message constant 'MESH_RESOURCE'."""
        return Metaclass_Marker.__constants['MESH_RESOURCE']

    @property
    def TRIANGLE_LIST(self):
        """Message constant 'TRIANGLE_LIST'."""
        return Metaclass_Marker.__constants['TRIANGLE_LIST']

    @property
    def ARROW_STRIP(self):
        """Message constant 'ARROW_STRIP'."""
        return Metaclass_Marker.__constants['ARROW_STRIP']

    @property
    def ADD(self):
        """Message constant 'ADD'."""
        return Metaclass_Marker.__constants['ADD']

    @property
    def MODIFY(self):
        """Message constant 'MODIFY'."""
        return Metaclass_Marker.__constants['MODIFY']

    @property
    def DELETE(self):
        """Message constant 'DELETE'."""
        return Metaclass_Marker.__constants['DELETE']

    @property
    def DELETEALL(self):
        """Message constant 'DELETEALL'."""
        return Metaclass_Marker.__constants['DELETEALL']


class Marker(metaclass=Metaclass_Marker):
    """
    Message class 'Marker'.

    Constants:
      ARROW
      CUBE
      SPHERE
      CYLINDER
      LINE_STRIP
      LINE_LIST
      CUBE_LIST
      SPHERE_LIST
      POINTS
      TEXT_VIEW_FACING
      MESH_RESOURCE
      TRIANGLE_LIST
      ARROW_STRIP
      ADD
      MODIFY
      DELETE
      DELETEALL
    """

    __slots__ = [
        '_header',
        '_ns',
        '_id',
        '_type',
        '_action',
        '_pose',
        '_scale',
        '_color',
        '_lifetime',
        '_frame_locked',
        '_points',
        '_colors',
        '_texture_resource',
        '_texture',
        '_uv_coordinates',
        '_text',
        '_mesh_resource',
        '_mesh_file',
        '_mesh_use_embedded_materials',
        '_check_fields',
    ]

    _fields_and_field_types = {
        'header': 'std_msgs/Header',
        'ns': 'string',
        'id': 'int32',
        'type': 'int32',
        'action': 'int32',
        'pose': 'geometry_msgs/Pose',
        'scale': 'geometry_msgs/Vector3',
        'color': 'std_msgs/ColorRGBA',
        'lifetime': 'builtin_interfaces/Duration',
        'frame_locked': 'boolean',
        'points': 'sequence<geometry_msgs/Point>',
        'colors': 'sequence<std_msgs/ColorRGBA>',
        'texture_resource': 'string',
        'texture': 'sensor_msgs/CompressedImage',
        'uv_coordinates': 'sequence<visualization_msgs/UVCoordinate>',
        'text': 'string',
        'mesh_resource': 'string',
        'mesh_file': 'visualization_msgs/MeshFile',
        'mesh_use_embedded_materials': 'boolean',
    }

    # This attribute is used to store an rosidl_parser.definition variable
    # related to the data type of each of the components the message.
    SLOT_TYPES = (
        rosidl_parser.definition.NamespacedType(['std_msgs', 'msg'], 'Header'),  # noqa: E501
        rosidl_parser.definition.UnboundedString(),  # noqa: E501
        rosidl_parser.definition.BasicType('int32'),  # noqa: E501
        rosidl_parser.definition.BasicType('int32'),  # noqa: E501
        rosidl_parser.definition.BasicType('int32'),  # noqa: E501
        rosidl_parser.definition.NamespacedType(['geometry_msgs', 'msg'], 'Pose'),  # noqa: E501
        rosidl_parser.definition.NamespacedType(['geometry_msgs', 'msg'], 'Vector3'),  # noqa: E501
        rosidl_parser.definition.NamespacedType(['std_msgs', 'msg'], 'ColorRGBA'),  # noqa: E501
        rosidl_parser.definition.NamespacedType(['builtin_interfaces', 'msg'], 'Duration'),  # noqa: E501
        rosidl_parser.definition.BasicType('boolean'),  # noqa: E501
        rosidl_parser.definition.UnboundedSequence(rosidl_parser.definition.NamespacedType(['geometry_msgs', 'msg'], 'Point')),  # noqa: E501
        rosidl_parser.definition.UnboundedSequence(rosidl_parser.definition.NamespacedType(['std_msgs', 'msg'], 'ColorRGBA')),  # noqa: E501
        rosidl_parser.definition.UnboundedString(),  # noqa: E501
        rosidl_parser.definition.NamespacedType(['sensor_msgs', 'msg'], 'CompressedImage'),  # noqa: E501
        rosidl_parser.definition.UnboundedSequence(rosidl_parser.definition.NamespacedType(['visualization_msgs', 'msg'], 'UVCoordinate')),  # noqa: E501
        rosidl_parser.definition.UnboundedString(),  # noqa: E501
        rosidl_parser.definition.UnboundedString(),  # noqa: E501
        rosidl_parser.definition.NamespacedType(['visualization_msgs', 'msg'], 'MeshFile'),  # noqa: E501
        rosidl_parser.definition.BasicType('boolean'),  # noqa: E501
    )

    def __init__(self, **kwargs):
        if 'check_fields' in kwargs:
            self._check_fields = kwargs['check_fields']
        else:
            self._check_fields = ros_python_check_fields == '1'
        if self._check_fields:
            assert all('_' + key in self.__slots__ for key in kwargs.keys()), \
                'Invalid arguments passed to constructor: %s' % \
                ', '.join(sorted(k for k in kwargs.keys() if '_' + k not in self.__slots__))
        from std_msgs.msg import Header
        self.header = kwargs.get('header', Header())
        self.ns = kwargs.get('ns', str())
        self.id = kwargs.get('id', int())
        self.type = kwargs.get('type', int())
        self.action = kwargs.get('action', int())
        from geometry_msgs.msg import Pose
        self.pose = kwargs.get('pose', Pose())
        from geometry_msgs.msg import Vector3
        self.scale = kwargs.get('scale', Vector3())
        from std_msgs.msg import ColorRGBA
        self.color = kwargs.get('color', ColorRGBA())
        from builtin_interfaces.msg import Duration
        self.lifetime = kwargs.get('lifetime', Duration())
        self.frame_locked = kwargs.get('frame_locked', bool())
        self.points = kwargs.get('points', [])
        self.colors = kwargs.get('colors', [])
        self.texture_resource = kwargs.get('texture_resource', str())
        from sensor_msgs.msg import CompressedImage
        self.texture = kwargs.get('texture', CompressedImage())
        self.uv_coordinates = kwargs.get('uv_coordinates', [])
        self.text = kwargs.get('text', str())
        self.mesh_resource = kwargs.get('mesh_resource', str())
        from visualization_msgs.msg import MeshFile
        self.mesh_file = kwargs.get('mesh_file', MeshFile())
        self.mesh_use_embedded_materials = kwargs.get('mesh_use_embedded_materials', bool())

    def __repr__(self):
        typename = self.__class__.__module__.split('.')
        typename.pop()
        typename.append(self.__class__.__name__)
        args = []
        for s, t in zip(self.get_fields_and_field_types().keys(), self.SLOT_TYPES):
            field = getattr(self, s)
            fieldstr = repr(field)
            # We use Python array type for fields that can be directly stored
            # in them, and "normal" sequences for everything else.  If it is
            # a type that we store in an array, strip off the 'array' portion.
            if (
                isinstance(t, rosidl_parser.definition.AbstractSequence) and
                isinstance(t.value_type, rosidl_parser.definition.BasicType) and
                t.value_type.typename in ['float', 'double', 'int8', 'uint8', 'int16', 'uint16', 'int32', 'uint32', 'int64', 'uint64']
            ):
                if len(field) == 0:
                    fieldstr = '[]'
                else:
                    if self._check_fields:
                        assert fieldstr.startswith('array(')
                    prefix = "array('X', "
                    suffix = ')'
                    fieldstr = fieldstr[len(prefix):-len(suffix)]
            args.append(s + '=' + fieldstr)
        return '%s(%s)' % ('.'.join(typename), ', '.join(args))

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return False
        if self.header != other.header:
            return False
        if self.ns != other.ns:
            return False
        if self.id != other.id:
            return False
        if self.type != other.type:
            return False
        if self.action != other.action:
            return False
        if self.pose != other.pose:
            return False
        if self.scale != other.scale:
            return False
        if self.color != other.color:
            return False
        if self.lifetime != other.lifetime:
            return False
        if self.frame_locked != other.frame_locked:
            return False
        if self.points != other.points:
            return False
        if self.colors != other.colors:
            return False
        if self.texture_resource != other.texture_resource:
            return False
        if self.texture != other.texture:
            return False
        if self.uv_coordinates != other.uv_coordinates:
            return False
        if self.text != other.text:
            return False
        if self.mesh_resource != other.mesh_resource:
            return False
        if self.mesh_file != other.mesh_file:
            return False
        if self.mesh_use_embedded_materials != other.mesh_use_embedded_materials:
            return False
        return True

    @classmethod
    def get_fields_and_field_types(cls):
        from copy import copy
        return copy(cls._fields_and_field_types)

    @builtins.property
    def header(self):
        """Message field 'header'."""
        return self._header

    @header.setter
    def header(self, value):
        if self._check_fields:
            from std_msgs.msg import Header
            assert \
                isinstance(value, Header), \
                "The 'header' field must be a sub message of type 'Header'"
        self._header = value

    @builtins.property
    def ns(self):
        """Message field 'ns'."""
        return self._ns

    @ns.setter
    def ns(self, value):
        if self._check_fields:
            assert \
                isinstance(value, str), \
                "The 'ns' field must be of type 'str'"
        self._ns = value

    @builtins.property  # noqa: A003
    def id(self):  # noqa: A003
        """Message field 'id'."""
        return self._id

    @id.setter  # noqa: A003
    def id(self, value):  # noqa: A003
        if self._check_fields:
            assert \
                isinstance(value, int), \
                "The 'id' field must be of type 'int'"
            assert value >= -2147483648 and value < 2147483648, \
                "The 'id' field must be an integer in [-2147483648, 2147483647]"
        self._id = value

    @builtins.property  # noqa: A003
    def type(self):  # noqa: A003
        """Message field 'type'."""
        return self._type

    @type.setter  # noqa: A003
    def type(self, value):  # noqa: A003
        if self._check_fields:
            assert \
                isinstance(value, int), \
                "The 'type' field must be of type 'int'"
            assert value >= -2147483648 and value < 2147483648, \
                "The 'type' field must be an integer in [-2147483648, 2147483647]"
        self._type = value

    @builtins.property
    def action(self):
        """Message field 'action'."""
        return self._action

    @action.setter
    def action(self, value):
        if self._check_fields:
            assert \
                isinstance(value, int), \
                "The 'action' field must be of type 'int'"
            assert value >= -2147483648 and value < 2147483648, \
                "The 'action' field must be an integer in [-2147483648, 2147483647]"
        self._action = value

    @builtins.property
    def pose(self):
        """Message field 'pose'."""
        return self._pose

    @pose.setter
    def pose(self, value):
        if self._check_fields:
            from geometry_msgs.msg import Pose
            assert \
                isinstance(value, Pose), \
                "The 'pose' field must be a sub message of type 'Pose'"
        self._pose = value

    @builtins.property
    def scale(self):
        """Message field 'scale'."""
        return self._scale

    @scale.setter
    def scale(self, value):
        if self._check_fields:
            from geometry_msgs.msg import Vector3
            assert \
                isinstance(value, Vector3), \
                "The 'scale' field must be a sub message of type 'Vector3'"
        self._scale = value

    @builtins.property
    def color(self):
        """Message field 'color'."""
        return self._color

    @color.setter
    def color(self, value):
        if self._check_fields:
            from std_msgs.msg import ColorRGBA
            assert \
                isinstance(value, ColorRGBA), \
                "The 'color' field must be a sub message of type 'ColorRGBA'"
        self._color = value

    @builtins.property
    def lifetime(self):
        """Message field 'lifetime'."""
        return self._lifetime

    @lifetime.setter
    def lifetime(self, value):
        if self._check_fields:
            from builtin_interfaces.msg import Duration
            assert \
                isinstance(value, Duration), \
                "The 'lifetime' field must be a sub message of type 'Duration'"
        self._lifetime = value

    @builtins.property
    def frame_locked(self):
        """Message field 'frame_locked'."""
        return self._frame_locked

    @frame_locked.setter
    def frame_locked(self, value):
        if self._check_fields:
            assert \
                isinstance(value, bool), \
                "The 'frame_locked' field must be of type 'bool'"
        self._frame_locked = value

    @builtins.property
    def points(self):
        """Message field 'points'."""
        return self._points

    @points.setter
    def points(self, value):
        if self._check_fields:
            from geometry_msgs.msg import Point
            from collections.abc import Sequence
            from collections.abc import Set
            from collections import UserList
            from collections import UserString
            assert \
                ((isinstance(value, Sequence) or
                  isinstance(value, Set) or
                  isinstance(value, UserList)) and
                 not isinstance(value, str) and
                 not isinstance(value, UserString) and
                 all(isinstance(v, Point) for v in value) and
                 True), \
                "The 'points' field must be a set or sequence and each value of type 'Point'"
        self._points = value

    @builtins.property
    def colors(self):
        """Message field 'colors'."""
        return self._colors

    @colors.setter
    def colors(self, value):
        if self._check_fields:
            from std_msgs.msg import ColorRGBA
            from collections.abc import Sequence
            from collections.abc import Set
            from collections import UserList
            from collections import UserString
            assert \
                ((isinstance(value, Sequence) or
                  isinstance(value, Set) or
                  isinstance(value, UserList)) and
                 not isinstance(value, str) and
                 not isinstance(value, UserString) and
                 all(isinstance(v, ColorRGBA) for v in value) and
                 True), \
                "The 'colors' field must be a set or sequence and each value of type 'ColorRGBA'"
        self._colors = value

    @builtins.property
    def texture_resource(self):
        """Message field 'texture_resource'."""
        return self._texture_resource

    @texture_resource.setter
    def texture_resource(self, value):
        if self._check_fields:
            assert \
                isinstance(value, str), \
                "The 'texture_resource' field must be of type 'str'"
        self._texture_resource = value

    @builtins.property
    def texture(self):
        """Message field 'texture'."""
        return self._texture

    @texture.setter
    def texture(self, value):
        if self._check_fields:
            from sensor_msgs.msg import CompressedImage
            assert \
                isinstance(value, CompressedImage), \
                "The 'texture' field must be a sub message of type 'CompressedImage'"
        self._texture = value

    @builtins.property
    def uv_coordinates(self):
        """Message field 'uv_coordinates'."""
        return self._uv_coordinates

    @uv_coordinates.setter
    def uv_coordinates(self, value):
        if self._check_fields:
            from visualization_msgs.msg import UVCoordinate
            from collections.abc import Sequence
            from collections.abc import Set
            from collections import UserList
            from collections import UserString
            assert \
                ((isinstance(value, Sequence) or
                  isinstance(value, Set) or
                  isinstance(value, UserList)) and
                 not isinstance(value, str) and
                 not isinstance(value, UserString) and
                 all(isinstance(v, UVCoordinate) for v in value) and
                 True), \
                "The 'uv_coordinates' field must be a set or sequence and each value of type 'UVCoordinate'"
        self._uv_coordinates = value

    @builtins.property
    def text(self):
        """Message field 'text'."""
        return self._text

    @text.setter
    def text(self, value):
        if self._check_fields:
            assert \
                isinstance(value, str), \
                "The 'text' field must be of type 'str'"
        self._text = value

    @builtins.property
    def mesh_resource(self):
        """Message field 'mesh_resource'."""
        return self._mesh_resource

    @mesh_resource.setter
    def mesh_resource(self, value):
        if self._check_fields:
            assert \
                isinstance(value, str), \
                "The 'mesh_resource' field must be of type 'str'"
        self._mesh_resource = value

    @builtins.property
    def mesh_file(self):
        """Message field 'mesh_file'."""
        return self._mesh_file

    @mesh_file.setter
    def mesh_file(self, value):
        if self._check_fields:
            from visualization_msgs.msg import MeshFile
            assert \
                isinstance(value, MeshFile), \
                "The 'mesh_file' field must be a sub message of type 'MeshFile'"
        self._mesh_file = value

    @builtins.property
    def mesh_use_embedded_materials(self):
        """Message field 'mesh_use_embedded_materials'."""
        return self._mesh_use_embedded_materials

    @mesh_use_embedded_materials.setter
    def mesh_use_embedded_materials(self, value):
        if self._check_fields:
            assert \
                isinstance(value, bool), \
                "The 'mesh_use_embedded_materials' field must be of type 'bool'"
        self._mesh_use_embedded_materials = value



# generated from rosidl_generator_py/resource/_idl.py.em
# with input from visualization_msgs:msg/InteractiveMarkerFeedback.idl
# generated code does not contain a copyright notice

# This is being done at the module level and not on the instance level to avoid looking
# for the same variable multiple times on each instance. This variable is not supposed to
# change during runtime so it makes sense to only look for it once.
from os import getenv

ros_python_check_fields = getenv('ROS_PYTHON_CHECK_FIELDS', default='')


# Import statements for member types

import builtins  # noqa: E402, I100

import rosidl_parser.definition  # noqa: E402, I100


class Metaclass_InteractiveMarkerFeedback(type):
    """Metaclass of message 'InteractiveMarkerFeedback'."""

    _CREATE_ROS_MESSAGE = None
    _CONVERT_FROM_PY = None
    _CONVERT_TO_PY = None
    _DESTROY_ROS_MESSAGE = None
    _TYPE_SUPPORT = None

    __constants = {
        'KEEP_ALIVE': 0,
        'POSE_UPDATE': 1,
        'MENU_SELECT': 2,
        'BUTTON_CLICK': 3,
        'MOUSE_DOWN': 4,
        'MOUSE_UP': 5,
    }

    @classmethod
    def __import_type_support__(cls):
        try:
            from rosidl_generator_py import import_type_support
            module = import_type_support('visualization_msgs')
        except ImportError:
            import logging
            import traceback
            logger = logging.getLogger(
                'visualization_msgs.msg.InteractiveMarkerFeedback')
            logger.debug(
                'Failed to import needed modules for type support:\n' +
                traceback.format_exc())
        else:
            cls._CREATE_ROS_MESSAGE = module.create_ros_message_msg__msg__interactive_marker_feedback
            cls._CONVERT_FROM_PY = module.convert_from_py_msg__msg__interactive_marker_feedback
            cls._CONVERT_TO_PY = module.convert_to_py_msg__msg__interactive_marker_feedback
            cls._TYPE_SUPPORT = module.type_support_msg__msg__interactive_marker_feedback
            cls._DESTROY_ROS_MESSAGE = module.destroy_ros_message_msg__msg__interactive_marker_feedback

            from geometry_msgs.msg import Point
            if Point.__class__._TYPE_SUPPORT is None:
                Point.__class__.__import_type_support__()

            from geometry_msgs.msg import Pose
            if Pose.__class__._TYPE_SUPPORT is None:
                Pose.__class__.__import_type_support__()

            from std_msgs.msg import Header
            if Header.__class__._TYPE_SUPPORT is None:
                Header.__class__.__import_type_support__()

    @classmethod
    def __prepare__(cls, name, bases, **kwargs):
        # list constant names here so that they appear in the help text of
        # the message class under "Data and other attributes defined here:"
        # as well as populate each message instance
        return {
            'KEEP_ALIVE': cls.__constants['KEEP_ALIVE'],
            'POSE_UPDATE': cls.__constants['POSE_UPDATE'],
            'MENU_SELECT': cls.__constants['MENU_SELECT'],
            'BUTTON_CLICK': cls.__constants['BUTTON_CLICK'],
            'MOUSE_DOWN': cls.__constants['MOUSE_DOWN'],
            'MOUSE_UP': cls.__constants['MOUSE_UP'],
        }

    @property
    def KEEP_ALIVE(self):
        """Message constant 'KEEP_ALIVE'."""
        return Metaclass_InteractiveMarkerFeedback.__constants['KEEP_ALIVE']

    @property
    def POSE_UPDATE(self):
        """Message constant 'POSE_UPDATE'."""
        return Metaclass_InteractiveMarkerFeedback.__constants['POSE_UPDATE']

    @property
    def MENU_SELECT(self):
        """Message constant 'MENU_SELECT'."""
        return Metaclass_InteractiveMarkerFeedback.__constants['MENU_SELECT']

    @property
    def BUTTON_CLICK(self):
        """Message constant 'BUTTON_CLICK'."""
        return Metaclass_InteractiveMarkerFeedback.__constants['BUTTON_CLICK']

    @property
    def MOUSE_DOWN(self):
        """Message constant 'MOUSE_DOWN'."""
        return Metaclass_InteractiveMarkerFeedback.__constants['MOUSE_DOWN']

    @property
    def MOUSE_UP(self):
        """Message constant 'MOUSE_UP'."""
        return Metaclass_InteractiveMarkerFeedback.__constants['MOUSE_UP']


class InteractiveMarkerFeedback(metaclass=Metaclass_InteractiveMarkerFeedback):
    """
    Message class 'InteractiveMarkerFeedback'.

    Constants:
      KEEP_ALIVE
      POSE_UPDATE
      MENU_SELECT
      BUTTON_CLICK
      MOUSE_DOWN
      MOUSE_UP
    """

    __slots__ = [
        '_header',
        '_client_id',
        '_marker_name',
        '_control_name',
        '_event_type',
        '_pose',
        '_menu_entry_id',
        '_mouse_point',
        '_mouse_point_valid',
        '_check_fields',
    ]

    _fields_and_field_types = {
        'header': 'std_msgs/Header',
        'client_id': 'string',
        'marker_name': 'string',
        'control_name': 'string',
        'event_type': 'uint8',
        'pose': 'geometry_msgs/Pose',
        'menu_entry_id': 'uint32',
        'mouse_point': 'geometry_msgs/Point',
        'mouse_point_valid': 'boolean',
    }

    # This attribute is used to store an rosidl_parser.definition variable
    # related to the data type of each of the components the message.
    SLOT_TYPES = (
        rosidl_parser.definition.NamespacedType(['std_msgs', 'msg'], 'Header'),  # noqa: E501
        rosidl_parser.definition.UnboundedString(),  # noqa: E501
        rosidl_parser.definition.UnboundedString(),  # noqa: E501
        rosidl_parser.definition.UnboundedString(),  # noqa: E501
        rosidl_parser.definition.BasicType('uint8'),  # noqa: E501
        rosidl_parser.definition.NamespacedType(['geometry_msgs', 'msg'], 'Pose'),  # noqa: E501
        rosidl_parser.definition.BasicType('uint32'),  # noqa: E501
        rosidl_parser.definition.NamespacedType(['geometry_msgs', 'msg'], 'Point'),  # noqa: E501
        rosidl_parser.definition.BasicType('boolean'),  # noqa: E501
    )

    def __init__(self, **kwargs):
        if 'check_fields' in kwargs:
            self._check_fields = kwargs['check_fields']
        else:
            self._check_fields = ros_python_check_fields == '1'
        if self._check_fields:
            assert all('_' + key in self.__slots__ for key in kwargs.keys()), \
                'Invalid arguments passed to constructor: %s' % \
                ', '.join(sorted(k for k in kwargs.keys() if '_' + k not in self.__slots__))
        from std_msgs.msg import Header
        self.header = kwargs.get('header', Header())
        self.client_id = kwargs.get('client_id', str())
        self.marker_name = kwargs.get('marker_name', str())
        self.control_name = kwargs.get('control_name', str())
        self.event_type = kwargs.get('event_type', int())
        from geometry_msgs.msg import Pose
        self.pose = kwargs.get('pose', Pose())
        self.menu_entry_id = kwargs.get('menu_entry_id', int())
        from geometry_msgs.msg import Point
        self.mouse_point = kwargs.get('mouse_point', Point())
        self.mouse_point_valid = kwargs.get('mouse_point_valid', bool())

    def __repr__(self):
        typename = self.__class__.__module__.split('.')
        typename.pop()
        typename.append(self.__class__.__name__)
        args = []
        for s, t in zip(self.get_fields_and_field_types().keys(), self.SLOT_TYPES):
            field = getattr(self, s)
            fieldstr = repr(field)
            # We use Python array type for fields that can be directly stored
            # in them, and "normal" sequences for everything else.  If it is
            # a type that we store in an array, strip off the 'array' portion.
            if (
                isinstance(t, rosidl_parser.definition.AbstractSequence) and
                isinstance(t.value_type, rosidl_parser.definition.BasicType) and
                t.value_type.typename in ['float', 'double', 'int8', 'uint8', 'int16', 'uint16', 'int32', 'uint32', 'int64', 'uint64']
            ):
                if len(field) == 0:
                    fieldstr = '[]'
                else:
                    if self._check_fields:
                        assert fieldstr.startswith('array(')
                    prefix = "array('X', "
                    suffix = ')'
                    fieldstr = fieldstr[len(prefix):-len(suffix)]
            args.append(s + '=' + fieldstr)
        return '%s(%s)' % ('.'.join(typename), ', '.join(args))

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return False
        if self.header != other.header:
            return False
        if self.client_id != other.client_id:
            return False
        if self.marker_name != other.marker_name:
            return False
        if self.control_name != other.control_name:
            return False
        if self.event_type != other.event_type:
            return False
        if self.pose != other.pose:
            return False
        if self.menu_entry_id != other.menu_entry_id:
            return False
        if self.mouse_point != other.mouse_point:
            return False
        if self.mouse_point_valid != other.mouse_point_valid:
            return False
        return True

    @classmethod
    def get_fields_and_field_types(cls):
        from copy import copy
        return copy(cls._fields_and_field_types)

    @builtins.property
    def header(self):
        """Message field 'header'."""
        return self._header

    @header.setter
    def header(self, value):
        if self._check_fields:
            from std_msgs.msg import Header
            assert \
                isinstance(value, Header), \
                "The 'header' field must be a sub message of type 'Header'"
        self._header = value

    @builtins.property
    def client_id(self):
        """Message field 'client_id'."""
        return self._client_id

    @client_id.setter
    def client_id(self, value):
        if self._check_fields:
            assert \
                isinstance(value, str), \
                "The 'client_id' field must be of type 'str'"
        self._client_id = value

    @builtins.property
    def marker_name(self):
        """Message field 'marker_name'."""
        return self._marker_name

    @marker_name.setter
    def marker_name(self, value):
        if self._check_fields:
            assert \
                isinstance(value, str), \
                "The 'marker_name' field must be of type 'str'"
        self._marker_name = value

    @builtins.property
    def control_name(self):
        """Message field 'control_name'."""
        return self._control_name

    @control_name.setter
    def control_name(self, value):
        if self._check_fields:
            assert \
                isinstance(value, str), \
                "The 'control_name' field must be of type 'str'"
        self._control_name = value

    @builtins.property
    def event_type(self):
        """Message field 'event_type'."""
        return self._event_type

    @event_type.setter
    def event_type(self, value):
        if self._check_fields:
            assert \
                isinstance(value, int), \
                "The 'event_type' field must be of type 'int'"
            assert value >= 0 and value < 256, \
                "The 'event_type' field must be an unsigned integer in [0, 255]"
        self._event_type = value

    @builtins.property
    def pose(self):
        """Message field 'pose'."""
        return self._pose

    @pose.setter
    def pose(self, value):
        if self._check_fields:
            from geometry_msgs.msg import Pose
            assert \
                isinstance(value, Pose), \
                "The 'pose' field must be a sub message of type 'Pose'"
        self._pose = value

    @builtins.property
    def menu_entry_id(self):
        """Message field 'menu_entry_id'."""
        return self._menu_entry_id

    @menu_entry_id.setter
    def menu_entry_id(self, value):
        if self._check_fields:
            assert \
                isinstance(value, int), \
                "The 'menu_entry_id' field must be of type 'int'"
            assert value >= 0 and value < 4294967296, \
                "The 'menu_entry_id' field must be an unsigned integer in [0, 4294967295]"
        self._menu_entry_id = value

    @builtins.property
    def mouse_point(self):
        """Message field 'mouse_point'."""
        return self._mouse_point

    @mouse_point.setter
    def mouse_point(self, value):
        if self._check_fields:
            from geometry_msgs.msg import Point
            assert \
                isinstance(value, Point), \
                "The 'mouse_point' field must be a sub message of type 'Point'"
        self._mouse_point = value

    @builtins.property
    def mouse_point_valid(self):
        """Message field 'mouse_point_valid'."""
        return self._mouse_point_valid

    @mouse_point_valid.setter
    def mouse_point_valid(self, value):
        if self._check_fields:
            assert \
                isinstance(value, bool), \
                "The 'mouse_point_valid' field must be of type 'bool'"
        self._mouse_point_valid = value



# generated from rosidl_generator_py/resource/_idl.py.em
# with input from visualization_msgs:msg/MenuEntry.idl
# generated code does not contain a copyright notice

# This is being done at the module level and not on the instance level to avoid looking
# for the same variable multiple times on each instance. This variable is not supposed to
# change during runtime so it makes sense to only look for it once.
from os import getenv

ros_python_check_fields = getenv('ROS_PYTHON_CHECK_FIELDS', default='')


# Import statements for member types

import builtins  # noqa: E402, I100

import rosidl_parser.definition  # noqa: E402, I100


class Metaclass_MenuEntry(type):
    """Metaclass of message 'MenuEntry'."""

    _CREATE_ROS_MESSAGE = None
    _CONVERT_FROM_PY = None
    _CONVERT_TO_PY = None
    _DESTROY_ROS_MESSAGE = None
    _TYPE_SUPPORT = None

    __constants = {
        'FEEDBACK': 0,
        'ROSRUN': 1,
        'ROSLAUNCH': 2,
    }

    @classmethod
    def __import_type_support__(cls):
        try:
            from rosidl_generator_py import import_type_support
            module = import_type_support('visualization_msgs')
        except ImportError:
            import logging
            import traceback
            logger = logging.getLogger(
                'visualization_msgs.msg.MenuEntry')
            logger.debug(
                'Failed to import needed modules for type support:\n' +
                traceback.format_exc())
        else:
            cls._CREATE_ROS_MESSAGE = module.create_ros_message_msg__msg__menu_entry
            cls._CONVERT_FROM_PY = module.convert_from_py_msg__msg__menu_entry
            cls._CONVERT_TO_PY = module.convert_to_py_msg__msg__menu_entry
            cls._TYPE_SUPPORT = module.type_support_msg__msg__menu_entry
            cls._DESTROY_ROS_MESSAGE = module.destroy_ros_message_msg__msg__menu_entry

    @classmethod
    def __prepare__(cls, name, bases, **kwargs):
        # list constant names here so that they appear in the help text of
        # the message class under "Data and other attributes defined here:"
        # as well as populate each message instance
        return {
            'FEEDBACK': cls.__constants['FEEDBACK'],
            'ROSRUN': cls.__constants['ROSRUN'],
            'ROSLAUNCH': cls.__constants['ROSLAUNCH'],
        }

    @property
    def FEEDBACK(self):
        """Message constant 'FEEDBACK'."""
        return Metaclass_MenuEntry.__constants['FEEDBACK']

    @property
    def ROSRUN(self):
        """Message constant 'ROSRUN'."""
        return Metaclass_MenuEntry.__constants['ROSRUN']

    @property
    def ROSLAUNCH(self):
        """Message constant 'ROSLAUNCH'."""
        return Metaclass_MenuEntry.__constants['ROSLAUNCH']


class MenuEntry(metaclass=Metaclass_MenuEntry):
    """
    Message class 'MenuEntry'.

    Constants:
      FEEDBACK
      ROSRUN
      ROSLAUNCH
    """

    __slots__ = [
        '_id',
        '_parent_id',
        '_title',
        '_command',
        '_command_type',
        '_check_fields',
    ]

    _fields_and_field_types = {
        'id': 'uint32',
        'parent_id': 'uint32',
        'title': 'string',
        'command': 'string',
        'command_type': 'uint8',
    }

    # This attribute is used to store an rosidl_parser.definition variable
    # related to the data type of each of the components the message.
    SLOT_TYPES = (
        rosidl_parser.definition.BasicType('uint32'),  # noqa: E501
        rosidl_parser.definition.BasicType('uint32'),  # noqa: E501
        rosidl_parser.definition.UnboundedString(),  # noqa: E501
        rosidl_parser.definition.UnboundedString(),  # noqa: E501
        rosidl_parser.definition.BasicType('uint8'),  # noqa: E501
    )

    def __init__(self, **kwargs):
        if 'check_fields' in kwargs:
            self._check_fields = kwargs['check_fields']
        else:
            self._check_fields = ros_python_check_fields == '1'
        if self._check_fields:
            assert all('_' + key in self.__slots__ for key in kwargs.keys()), \
                'Invalid arguments passed to constructor: %s' % \
                ', '.join(sorted(k for k in kwargs.keys() if '_' + k not in self.__slots__))
        self.id = kwargs.get('id', int())
        self.parent_id = kwargs.get('parent_id', int())
        self.title = kwargs.get('title', str())
        self.command = kwargs.get('command', str())
        self.command_type = kwargs.get('command_type', int())

    def __repr__(self):
        typename = self.__class__.__module__.split('.')
        typename.pop()
        typename.append(self.__class__.__name__)
        args = []
        for s, t in zip(self.get_fields_and_field_types().keys(), self.SLOT_TYPES):
            field = getattr(self, s)
            fieldstr = repr(field)
            # We use Python array type for fields that can be directly stored
            # in them, and "normal" sequences for everything else.  If it is
            # a type that we store in an array, strip off the 'array' portion.
            if (
                isinstance(t, rosidl_parser.definition.AbstractSequence) and
                isinstance(t.value_type, rosidl_parser.definition.BasicType) and
                t.value_type.typename in ['float', 'double', 'int8', 'uint8', 'int16', 'uint16', 'int32', 'uint32', 'int64', 'uint64']
            ):
                if len(field) == 0:
                    fieldstr = '[]'
                else:
                    if self._check_fields:
                        assert fieldstr.startswith('array(')
                    prefix = "array('X', "
                    suffix = ')'
                    fieldstr = fieldstr[len(prefix):-len(suffix)]
            args.append(s + '=' + fieldstr)
        return '%s(%s)' % ('.'.join(typename), ', '.join(args))

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return False
        if self.id != other.id:
            return False
        if self.parent_id != other.parent_id:
            return False
        if self.title != other.title:
            return False
        if self.command != other.command:
            return False
        if self.command_type != other.command_type:
            return False
        return True

    @classmethod
    def get_fields_and_field_types(cls):
        from copy import copy
        return copy(cls._fields_and_field_types)

    @builtins.property  # noqa: A003
    def id(self):  # noqa: A003
        """Message field 'id'."""
        return self._id

    @id.setter  # noqa: A003
    def id(self, value):  # noqa: A003
        if self._check_fields:
            assert \
                isinstance(value, int), \
                "The 'id' field must be of type 'int'"
            assert value >= 0 and value < 4294967296, \
                "The 'id' field must be an unsigned integer in [0, 4294967295]"
        self._id = value

    @builtins.property
    def parent_id(self):
        """Message field 'parent_id'."""
        return self._parent_id

    @parent_id.setter
    def parent_id(self, value):
        if self._check_fields:
            assert \
                isinstance(value, int), \
                "The 'parent_id' field must be of type 'int'"
            assert value >= 0 and value < 4294967296, \
                "The 'parent_id' field must be an unsigned integer in [0, 4294967295]"
        self._parent_id = value

    @builtins.property
    def title(self):
        """Message field 'title'."""
        return self._title

    @title.setter
    def title(self, value):
        if self._check_fields:
            assert \
                isinstance(value, str), \
                "The 'title' field must be of type 'str'"
        self._title = value

    @builtins.property
    def command(self):
        """Message field 'command'."""
        return self._command

    @command.setter
    def command(self, value):
        if self._check_fields:
            assert \
                isinstance(value, str), \
                "The 'command' field must be of type 'str'"
        self._command = value

    @builtins.property
    def command_type(self):
        """Message field 'command_type'."""
        return self._command_type

    @command_type.setter
    def command_type(self, value):
        if self._check_fields:
            assert \
                isinstance(value, int), \
                "The 'command_type' field must be of type 'int'"
            assert value >= 0 and value < 256, \
                "The 'command_type' field must be an unsigned integer in [0, 255]"
        self._command_type = value



# generated from rosidl_generator_py/resource/_idl.py.em
# with input from visualization_msgs:msg/InteractiveMarkerControl.idl
# generated code does not contain a copyright notice

# This is being done at the module level and not on the instance level to avoid looking
# for the same variable multiple times on each instance. This variable is not supposed to
# change during runtime so it makes sense to only look for it once.
from os import getenv

ros_python_check_fields = getenv('ROS_PYTHON_CHECK_FIELDS', default='')


# Import statements for member types

import builtins  # noqa: E402, I100

import rosidl_parser.definition  # noqa: E402, I100


class Metaclass_InteractiveMarkerControl(type):
    """Metaclass of message 'InteractiveMarkerControl'."""

    _CREATE_ROS_MESSAGE = None
    _CONVERT_FROM_PY = None
    _CONVERT_TO_PY = None
    _DESTROY_ROS_MESSAGE = None
    _TYPE_SUPPORT = None

    __constants = {
        'INHERIT': 0,
        'FIXED': 1,
        'VIEW_FACING': 2,
        'NONE': 0,
        'MENU': 1,
        'BUTTON': 2,
        'MOVE_AXIS': 3,
        'MOVE_PLANE': 4,
        'ROTATE_AXIS': 5,
        'MOVE_ROTATE': 6,
        'MOVE_3D': 7,
        'ROTATE_3D': 8,
        'MOVE_ROTATE_3D': 9,
    }

    @classmethod
    def __import_type_support__(cls):
        try:
            from rosidl_generator_py import import_type_support
            module = import_type_support('visualization_msgs')
        except ImportError:
            import logging
            import traceback
            logger = logging.getLogger(
                'visualization_msgs.msg.InteractiveMarkerControl')
            logger.debug(
                'Failed to import needed modules for type support:\n' +
                traceback.format_exc())
        else:
            cls._CREATE_ROS_MESSAGE = module.create_ros_message_msg__msg__interactive_marker_control
            cls._CONVERT_FROM_PY = module.convert_from_py_msg__msg__interactive_marker_control
            cls._CONVERT_TO_PY = module.convert_to_py_msg__msg__interactive_marker_control
            cls._TYPE_SUPPORT = module.type_support_msg__msg__interactive_marker_control
            cls._DESTROY_ROS_MESSAGE = module.destroy_ros_message_msg__msg__interactive_marker_control

            from geometry_msgs.msg import Quaternion
            if Quaternion.__class__._TYPE_SUPPORT is None:
                Quaternion.__class__.__import_type_support__()

            from visualization_msgs.msg import Marker
            if Marker.__class__._TYPE_SUPPORT is None:
                Marker.__class__.__import_type_support__()

    @classmethod
    def __prepare__(cls, name, bases, **kwargs):
        # list constant names here so that they appear in the help text of
        # the message class under "Data and other attributes defined here:"
        # as well as populate each message instance
        return {
            'INHERIT': cls.__constants['INHERIT'],
            'FIXED': cls.__constants['FIXED'],
            'VIEW_FACING': cls.__constants['VIEW_FACING'],
            'NONE': cls.__constants['NONE'],
            'MENU': cls.__constants['MENU'],
            'BUTTON': cls.__constants['BUTTON'],
            'MOVE_AXIS': cls.__constants['MOVE_AXIS'],
            'MOVE_PLANE': cls.__constants['MOVE_PLANE'],
            'ROTATE_AXIS': cls.__constants['ROTATE_AXIS'],
            'MOVE_ROTATE': cls.__constants['MOVE_ROTATE'],
            'MOVE_3D': cls.__constants['MOVE_3D'],
            'ROTATE_3D': cls.__constants['ROTATE_3D'],
            'MOVE_ROTATE_3D': cls.__constants['MOVE_ROTATE_3D'],
        }

    @property
    def INHERIT(self):
        """Message constant 'INHERIT'."""
        return Metaclass_InteractiveMarkerControl.__constants['INHERIT']

    @property
    def FIXED(self):
        """Message constant 'FIXED'."""
        return Metaclass_InteractiveMarkerControl.__constants['FIXED']

    @property
    def VIEW_FACING(self):
        """Message constant 'VIEW_FACING'."""
        return Metaclass_InteractiveMarkerControl.__constants['VIEW_FACING']

    @property
    def NONE(self):
        """Message constant 'NONE'."""
        return Metaclass_InteractiveMarkerControl.__constants['NONE']

    @property
    def MENU(self):
        """Message constant 'MENU'."""
        return Metaclass_InteractiveMarkerControl.__constants['MENU']

    @property
    def BUTTON(self):
        """Message constant 'BUTTON'."""
        return Metaclass_InteractiveMarkerControl.__constants['BUTTON']

    @property
    def MOVE_AXIS(self):
        """Message constant 'MOVE_AXIS'."""
        return Metaclass_InteractiveMarkerControl.__constants['MOVE_AXIS']

    @property
    def MOVE_PLANE(self):
        """Message constant 'MOVE_PLANE'."""
        return Metaclass_InteractiveMarkerControl.__constants['MOVE_PLANE']

    @property
    def ROTATE_AXIS(self):
        """Message constant 'ROTATE_AXIS'."""
        return Metaclass_InteractiveMarkerControl.__constants['ROTATE_AXIS']

    @property
    def MOVE_ROTATE(self):
        """Message constant 'MOVE_ROTATE'."""
        return Metaclass_InteractiveMarkerControl.__constants['MOVE_ROTATE']

    @property
    def MOVE_3D(self):
        """Message constant 'MOVE_3D'."""
        return Metaclass_InteractiveMarkerControl.__constants['MOVE_3D']

    @property
    def ROTATE_3D(self):
        """Message constant 'ROTATE_3D'."""
        return Metaclass_InteractiveMarkerControl.__constants['ROTATE_3D']

    @property
    def MOVE_ROTATE_3D(self):
        """Message constant 'MOVE_ROTATE_3D'."""
        return Metaclass_InteractiveMarkerControl.__constants['MOVE_ROTATE_3D']


class InteractiveMarkerControl(metaclass=Metaclass_InteractiveMarkerControl):
    """
    Message class 'InteractiveMarkerControl'.

    Constants:
      INHERIT
      FIXED
      VIEW_FACING
      NONE
      MENU
      BUTTON
      MOVE_AXIS
      MOVE_PLANE
      ROTATE_AXIS
      MOVE_ROTATE
      MOVE_3D
      ROTATE_3D
      MOVE_ROTATE_3D
    """

    __slots__ = [
        '_name',
        '_orientation',
        '_orientation_mode',
        '_interaction_mode',
        '_always_visible',
        '_markers',
        '_independent_marker_orientation',
        '_description',
        '_check_fields',
    ]

    _fields_and_field_types = {
        'name': 'string',
        'orientation': 'geometry_msgs/Quaternion',
        'orientation_mode': 'uint8',
        'interaction_mode': 'uint8',
        'always_visible': 'boolean',
        'markers': 'sequence<visualization_msgs/Marker>',
        'independent_marker_orientation': 'boolean',
        'description': 'string',
    }

    # This attribute is used to store an rosidl_parser.definition variable
    # related to the data type of each of the components the message.
    SLOT_TYPES = (
        rosidl_parser.definition.UnboundedString(),  # noqa: E501
        rosidl_parser.definition.NamespacedType(['geometry_msgs', 'msg'], 'Quaternion'),  # noqa: E501
        rosidl_parser.definition.BasicType('uint8'),  # noqa: E501
        rosidl_parser.definition.BasicType('uint8'),  # noqa: E501
        rosidl_parser.definition.BasicType('boolean'),  # noqa: E501
        rosidl_parser.definition.UnboundedSequence(rosidl_parser.definition.NamespacedType(['visualization_msgs', 'msg'], 'Marker')),  # noqa: E501
        rosidl_parser.definition.BasicType('boolean'),  # noqa: E501
        rosidl_parser.definition.UnboundedString(),  # noqa: E501
    )

    def __init__(self, **kwargs):
        if 'check_fields' in kwargs:
            self._check_fields = kwargs['check_fields']
        else:
            self._check_fields = ros_python_check_fields == '1'
        if self._check_fields:
            assert all('_' + key in self.__slots__ for key in kwargs.keys()), \
                'Invalid arguments passed to constructor: %s' % \
                ', '.join(sorted(k for k in kwargs.keys() if '_' + k not in self.__slots__))
        self.name = kwargs.get('name', str())
        from geometry_msgs.msg import Quaternion
        self.orientation = kwargs.get('orientation', Quaternion())
        self.orientation_mode = kwargs.get('orientation_mode', int())
        self.interaction_mode = kwargs.get('interaction_mode', int())
        self.always_visible = kwargs.get('always_visible', bool())
        self.markers = kwargs.get('markers', [])
        self.independent_marker_orientation = kwargs.get('independent_marker_orientation', bool())
        self.description = kwargs.get('description', str())

    def __repr__(self):
        typename = self.__class__.__module__.split('.')
        typename.pop()
        typename.append(self.__class__.__name__)
        args = []
        for s, t in zip(self.get_fields_and_field_types().keys(), self.SLOT_TYPES):
            field = getattr(self, s)
            fieldstr = repr(field)
            # We use Python array type for fields that can be directly stored
            # in them, and "normal" sequences for everything else.  If it is
            # a type that we store in an array, strip off the 'array' portion.
            if (
                isinstance(t, rosidl_parser.definition.AbstractSequence) and
                isinstance(t.value_type, rosidl_parser.definition.BasicType) and
                t.value_type.typename in ['float', 'double', 'int8', 'uint8', 'int16', 'uint16', 'int32', 'uint32', 'int64', 'uint64']
            ):
                if len(field) == 0:
                    fieldstr = '[]'
                else:
                    if self._check_fields:
                        assert fieldstr.startswith('array(')
                    prefix = "array('X', "
                    suffix = ')'
                    fieldstr = fieldstr[len(prefix):-len(suffix)]
            args.append(s + '=' + fieldstr)
        return '%s(%s)' % ('.'.join(typename), ', '.join(args))

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return False
        if self.name != other.name:
            return False
        if self.orientation != other.orientation:
            return False
        if self.orientation_mode != other.orientation_mode:
            return False
        if self.interaction_mode != other.interaction_mode:
            return False
        if self.always_visible != other.always_visible:
            return False
        if self.markers != other.markers:
            return False
        if self.independent_marker_orientation != other.independent_marker_orientation:
            return False
        if self.description != other.description:
            return False
        return True

    @classmethod
    def get_fields_and_field_types(cls):
        from copy import copy
        return copy(cls._fields_and_field_types)

    @builtins.property
    def name(self):
        """Message field 'name'."""
        return self._name

    @name.setter
    def name(self, value):
        if self._check_fields:
            assert \
                isinstance(value, str), \
                "The 'name' field must be of type 'str'"
        self._name = value

    @builtins.property
    def orientation(self):
        """Message field 'orientation'."""
        return self._orientation

    @orientation.setter
    def orientation(self, value):
        if self._check_fields:
            from geometry_msgs.msg import Quaternion
            assert \
                isinstance(value, Quaternion), \
                "The 'orientation' field must be a sub message of type 'Quaternion'"
        self._orientation = value

    @builtins.property
    def orientation_mode(self):
        """Message field 'orientation_mode'."""
        return self._orientation_mode

    @orientation_mode.setter
    def orientation_mode(self, value):
        if self._check_fields:
            assert \
                isinstance(value, int), \
                "The 'orientation_mode' field must be of type 'int'"
            assert value >= 0 and value < 256, \
                "The 'orientation_mode' field must be an unsigned integer in [0, 255]"
        self._orientation_mode = value

    @builtins.property
    def interaction_mode(self):
        """Message field 'interaction_mode'."""
        return self._interaction_mode

    @interaction_mode.setter
    def interaction_mode(self, value):
        if self._check_fields:
            assert \
                isinstance(value, int), \
                "The 'interaction_mode' field must be of type 'int'"
            assert value >= 0 and value < 256, \
                "The 'interaction_mode' field must be an unsigned integer in [0, 255]"
        self._interaction_mode = value

    @builtins.property
    def always_visible(self):
        """Message field 'always_visible'."""
        return self._always_visible

    @always_visible.setter
    def always_visible(self, value):
        if self._check_fields:
            assert \
                isinstance(value, bool), \
                "The 'always_visible' field must be of type 'bool'"
        self._always_visible = value

    @builtins.property
    def markers(self):
        """Message field 'markers'."""
        return self._markers

    @markers.setter
    def markers(self, value):
        if self._check_fields:
            from visualization_msgs.msg import Marker
            from collections.abc import Sequence
            from collections.abc import Set
            from collections import UserList
            from collections import UserString
            assert \
                ((isinstance(value, Sequence) or
                  isinstance(value, Set) or
                  isinstance(value, UserList)) and
                 not isinstance(value, str) and
                 not isinstance(value, UserString) and
                 all(isinstance(v, Marker) for v in value) and
                 True), \
                "The 'markers' field must be a set or sequence and each value of type 'Marker'"
        self._markers = value

    @builtins.property
    def independent_marker_orientation(self):
        """Message field 'independent_marker_orientation'."""
        return self._independent_marker_orientation

    @independent_marker_orientation.setter
    def independent_marker_orientation(self, value):
        if self._check_fields:
            assert \
                isinstance(value, bool), \
                "The 'independent_marker_orientation' field must be of type 'bool'"
        self._independent_marker_orientation = value

    @builtins.property
    def description(self):
        """Message field 'description'."""
        return self._description

    @description.setter
    def description(self, value):
        if self._check_fields:
            assert \
                isinstance(value, str), \
                "The 'description' field must be of type 'str'"
        self._description = value
